@@ -2,7 +2,6 @@ use strict;
 use warnings;
 package App::DubiousHTTP::Tests::Broken;
 use App::DubiousHTTP::Tests::Common;
-use Compress::Raw::Zlib;
 
 SETUP( 
     'broken',
@@ -20,6 +19,18 @@ DESC
 
     [ MUSTBE_VALID,  'ok' => 'VALID: simple request with content-length'],
     [ UNCOMMON_VALID, 'http09' => 'HTTP 0.9 response (no header)'],
+    [ INVALID, '\012http09' => 'HTTP 0.9 response (no header) prefix with \n'],
+    [ INVALID, '\012\012http09' => 'HTTP 0.9 response (no header) prefix with \n\n'],
+    [ INVALID, 'HTT\012\012http09' => 'response prefixed with HTT\n\n'],
+    [ INVALID, 'HTTP\012\012http09' => 'response prefixed with HTTP\n\n'],
+    [ INVALID, 'hTtp\012\012http09' => 'response prefixed with hTtp\n\n'],
+    [ INVALID, 'HTTP\012http09' => 'response prefixed with HTTP\n'],
+    [ INVALID, 'hTtp\012http09' => 'response prefixed with hTtp\n'],
+    [ INVALID, 'HTTP/\012\012http09' => 'response prefixed with HTTP/\n\n'],
+    [ INVALID, 'HTTP.\012\012http09' => 'response prefixed with HTTP.\n\n'],
+    [ INVALID, 'HTTPx\012\012http09' => 'response prefixed with HTTPx\n\n'],
+    [ INVALID, 'HTTP/1.1\040100\040ok\012\012http09' => 'response prefixed with HTTP/1.1 100 ok\n\n'],
+    [ INVALID, 'HTTP/1.1\040\053100\040ok\012\012http09' => 'response prefixed with HTTP/1.1 +100 ok\n\n'],
 
     [ 'INVALID: junk data around transfer-encoding' ],
     [ INVALID, 'chunked;177' => 'Transfer-Encoding: chunked\r\n\177\r\n, served chunked' ],
@@ -64,6 +75,7 @@ DESC
     [ INVALID, 'data:Transfer\000-encoding:chunked\015\012;do_chunked' => '"Transfer\000-Encoding:chunked", served chunked' ],
     [ INVALID, 'data:Transfer\000-encoding:chun\000ked\015\012;do_chunked' => '"Transfer\000-Encoding:chun\000ked", served chunked' ],
     [ INVALID, 'te\000:chunked;do_chunked' => '"Transfer-Encoding\000:chunked", served chunked' ],
+    [ INVALID, 'te\040\011\040\011\040:chunked;do_chunked' => '"Transfer-Encoding \t \t :chunked", served chunked' ],
     [ INVALID, 'te\013:chunked;do_chunked' => '"Transfer-Encoding\v:chunked", served chunked' ],
     [ INVALID, 'te\014:chunked;do_chunked' => '"Transfer-Encoding\f:chunked", served chunked' ],
     [ INVALID, 'te\012\040:chunked;do_chunked' => '"Transfer-Encoding\n :chunked", served chunked' ],
@@ -167,6 +179,22 @@ DESC
     [ INVALID, 'proto:HTTP/1.1-lf' => 'HTTP/1.1+LF: \n after version in status line'],
     [ INVALID, "proto:space-HTTP/1.1" => 'version prefixed with space: SPACE+HTTP/1.1'],
     [ INVALID, 'proto:FTP/1.1' => 'version FTP/1.1 instead of HTTP/1.1'],
+    [ INVALID, 'proto:ICY' => 'version ICY instead of HTTP/1.0'],
+    [ INVALID, 'proto:ICY;gzip' => 'version ICY instead of HTTP/1.0 compressed with gzip'],
+    [ INVALID, 'proto:HTTP\1.1' => 'version HTTP\1.1 instead of HTTP/1.1'],
+    [ INVALID, 'proto:HTTP/1.010;chunked' => 'version HTTP/1.010 instead of HTTP/1.1 and chunked'],
+    [ INVALID, 'proto:HTTP/1.010;chunked;do_clen' => 'version HTTP/0.010 instead of HTTP/1.1 and TE chunked but not served chunked'],
+    [ INVALID, 'proto:HTTP/+1.+1;chunked' => 'version HTTP/+1.+1 instead of HTTP/1.1 and chunked'],
+    [ INVALID, 'proto:HTTP/+1.+1;chunked;do_clen' => 'version HTTP/+1.+1 instead of HTTP/1.1 and TE chunked but not served chunked'],
+    [ INVALID, 'proto:HTTP/1A.1B;chunked' => 'version HTTP/1A.1B instead of HTTP/1.1 and chunked'],
+    [ INVALID, 'proto:HTTP/1A.1B;chunked;do_clen' => 'version HTTP/1A.1B instead of HTTP/1.1 and TE chunked but not served chunked'],
+    [ INVALID, 'proto:HTTP/2.B;chunked' => 'version HTTP/2.B instead of HTTP/1.1 and chunked'],
+    [ INVALID, 'proto:HTTP/2.B;chunked;do_clen' => 'version HTTP/2.B instead of HTTP/1.1 and TE chunked but not served chunked'],
+    [ INVALID, 'proto:HTTP/9.-1;chunked' => 'version HTTP/9.-1 instead of HTTP/1.1 and chunked'],
+    [ INVALID, 'proto:HTTP/9.-1;chunked;do_clen' => 'version HTTP/9.-1 instead of HTTP/1.1 and TE chunked but not served chunked'],
+    [ INVALID, 'proto:HTTP/1\040.1;chunked' => 'version HTTP/1<space>.1 instead of HTTP/1.1 and chunked'],
+    [ INVALID, 'proto:HTTP/1\040.1;chunked;do_clen' => 'version HTTP/1<space>.1 instead of HTTP/1.1 and TE chunked but not served chunked'],
+    [ INVALID, 'proto:HTTP/A.B;gzip' => 'version HTTP/A.B instead of HTTP/1.0 and gzipped'],
     [ INVALID, 'status:HTTP/1.1' => 'HTTP/1.1 without code'],
     [ INVALID, 'status:HTTP/1.1(cr)Transfer-Encoding:chunked;do_clen' => 'HTTP/1.1\rTransfer-Encoding:chunked, not served chunked'],
     [ INVALID, 'status:HTTP/1.1(cr)Transfer-Encoding:chunked;do_chunked' => 'HTTP/1.1\rTransfer-Encoding:chunked, served chunked'],
@@ -194,6 +222,7 @@ DESC
     [ INVALID, 'status:HTTP/1.1(space)200\000(space)ok;chunked' => 'HTTP/1.1 200\000 ok\r\n and chunked'],
     [ INVALID, 'status:HTTP/1.1\000200\000ok;chunked' => 'HTTP/1.1\000200\000ok\r\n and chunked'],
     [ INVALID, 'status:HTTP/1.1\013200\013ok;chunked' => 'HTTP/1.1\013200\013ok\r\n and chunked'],
+    [ INVALID, 'status:HTTP/1.1(space)-65336(space);chunked' => 'HTTP/1.1 -65336 ok\r\n and chunked'], # uint_16 -> 200
     [ INVALID, 'status:HTTP/1.1foobar;chunked' => 'HTTP/1.1foobar\r\n and chunked'],
     [ INVALID, 'status:HTTP/1.1foobar(cr)Transfer-Encoding:chunked;do_chunked' => 'HTTP/1.1foobar\r and chunked'],
     [ INVALID, 'status:HTTP/1.foobar;chunked' => 'HTTP/1.foobar\r\n and chunked'],
@@ -201,6 +230,11 @@ DESC
     [ INVALID, 'status:HTTP/foobar;chunked' => 'HTTP/foobar\r\n and chunked'],
     [ INVALID, 'status:HTTP/;chunked' => 'HTTP/\r\n and chunked'],
     [ INVALID, 'status:HTTP;chunked' => 'HTTP\r\n and chunked'],
+    [ INVALID, 'status:HTTP/1.1\011204(space)ok;chunked' => 'HTTP/1.1\t204 ok with chunked content'],
+    [ INVALID, 'status:HTTP/1.1\011304(space)ok;chunked' => 'HTTP/1.1\t304 ok with chunked content'],
+    [ INVALID, 'status:HTTP/1.1\040\040\040\040204(space)ok;chunked' => 'HTTP/1.1    204 ok with chunked content'],
+    [ INVALID, 'status:HTTP/1.1\040\040\040\040304(space)ok;chunked' => 'HTTP/1.1    304 ok with chunked content'],
+    [ INVALID, 'status:HTTP\1.1(space)200(space)ok;chunked' => 'HTTP\1.1 instead of HTTP/1.1 with chunked content'],
     [ INVALID, 'status:Transfer-Encoding:chunked;do_clen' => 'no status line but Transfer-Encoding:chunked, not served chunked'],
     [ INVALID, 'status:Transfer-Encoding:chunked;do_chunked' => 'no status line but Transfer-Encoding:chunked, served chunked'],
     [ INVALID, 'cr-no-crlf' => 'single \r instead of \r\n' ],
@@ -219,6 +253,7 @@ DESC
     [ INVALID, 'chunked;lfcr-no-crlf' => '\n\r instead of \r\n and chunked' ],
     [ INVALID, 'chunked;lfcr-no-crlf;end-crlfcrlf' => '\n\r instead of \r\n, but end \r\n\r\n and chunked' ],
     [ INVALID, 'chunked;cr\000lf-no-crlf' => '\r\000\n instead of \r\n and chunked' ],
+    [ INVALID, 'chunked;lfcrcrlf-no-crlf;end-crlfcrlf' => '\n\r\r\n instead of \r\n as line delimiter, but end \r\n\r\n and chunked' ],
     [ INVALID, 'end-crcr' => 'header end \r\r' ],
     [ INVALID, 'end-lflf' => 'header end \n\n' ],
     [ INVALID, 'end-crlf\000crlf' => 'header end \r\n\000\r\n' ],
@@ -292,6 +327,10 @@ DESC
     [ UNCOMMON_VALID, '406' => 'code 406 with body'],
     [ UNCOMMON_VALID, '500' => 'code 500 with body'],
     [ UNCOMMON_VALID, '502' => 'code 502 with body'],
+    [ UNCOMMON_INVALID, '100+' => 'code 100 followed by real response'],
+    [ INVALID, '100+b' => 'code 100 with body followed by real response'],
+    [ INVALID, '16-100+' => 'code -65436(100) followed by real response'],
+    [ INVALID, '16-100+b' => 'code -65436(100) with body followed by real response'],
 
     [ 'VALID: non-existing status codes' ],
     [ INVALID, '000' => 'code 000 with body'],
@@ -315,10 +354,14 @@ DESC
     [ INVALID, '204;chunked' => 'code 204 with chunked body'],
     [ INVALID, '0204' => 'code 0204 with body'],
     [ INVALID, '2040' => 'code 2040 with body'],
+    [ INVALID, '304;chunked' => 'code 304 with chunked body'],
+    [ INVALID, '0304' => 'code 0304 with body'],
+    [ INVALID, '3040' => 'code 3040 with body'],
 
     [ 'VALID: new lines before HTTP header' ],
     [ UNCOMMON_VALID, 'crlf-header;chunked' => 'single <CR><LF> before header, chunked'],
     [ UNCOMMON_VALID, 'crlf-crlf-header;chunked' => 'double <CR><LF> before header, chunked'],
+    [ 'INVALID: other stuff before HTTP header' ],
     [ INVALID, 'space-crlf-header;chunked' => 'space followed by single <CR><LF> before header, chunked'],
     [ INVALID, 'space-tab-crlf-header;chunked' => 'space+TAB followed by single <CR><LF> before header, chunked'],
     [ INVALID, 'space-crlf-tab-crlf-header;chunked' => 'space+CRLF+TAB+CRLF before header, chunked'],
@@ -334,6 +377,12 @@ DESC
     [ INVALID, 'lf-lf-header;chunked' => 'double <LF> before header, chunked'],
     [ INVALID, 'space-lf-header;chunked' => 'space followed by single <LF> before header, chunked'],
     [ INVALID, 'lfcr-header;chunked' => 'single <LF><CR> before header, chunked'],
+    # chrome accepts up to 4 bytes garbage before HTTP/
+    [ INVALID, 'H-header;chunked' => '"H"  before header, chunked'],
+    [ INVALID, 'HT-header;chunked' => '"HT"  before header, chunked'],
+    [ INVALID, 'HTT-header;chunked' => '"HTT"  before header, chunked'],
+    [ INVALID, 'HTTX-header;chunked' => '"HTTX"  before header, chunked'],
+    [ INVALID, 'HTTXY-header;chunked' => '"HTTXY"  before header, chunked'],
 );
 
 sub make_response {
@@ -377,9 +426,9 @@ sub make_response {
 	    $te = 'chunked';
 	    $hdr .= "Transfer-Encoding: chunked\r\n";
 	} elsif ( m{^do_(gzip|deflate)} ) {
-	    $data = _compress($data,$1);
+	    $data = zlib_compress($data,$1);
 	} elsif ( m{^(gzip|deflate)$} ) {
-	    $data = _compress($data,$1);
+	    $data = zlib_compress($data,$1);
 	    $hdr .= "Content-Encoding: $1\r\n";
 	} elsif ( $_ eq 'do_clen') {
 	    $te = 'clen';
@@ -391,8 +440,24 @@ sub make_response {
 	    $hdr .= "\177\r\n";
 	} elsif ( $_ eq 'only' ) {
 	    $only = 1;
-	} elsif ( $_ eq 'http09' ) {
-	    return $data;
+	} elsif ( s{\Aprefix:((?:\w+|\\[0-7]{3})+)\z}{$1} ) {
+	    s{\\([0-7]{3})}{ chr(oct($1)) }eg;
+	    $prefix = $_;
+	} elsif ( m{\A((?:[\w/.]+|\\[0-7]{3})*)http09\z} ) {
+	    my $prefix = $1 or return $data;
+	    $prefix =~s{\\([0-7]{3})}{ chr(oct($1)) }eg;
+	    return $prefix . $data;
+	} elsif ( m{^(16-)?(\d+)\+(b?)\z}) { # 100+
+	    my $code = $1 ? "-".(65536-$2):$2;
+	    $prefix = "HTTP/1.1 $code whatever\r\n";
+	    if ($3) {
+		my $body = "fooo";
+		$prefix .= "Content-length: ".length($body)."\r\n";
+		$prefix .= "\r\n";
+		$prefix .= $body;
+	    } else {
+		$prefix .= "\r\n";
+	    }
 	} elsif ( m{^(space-|tab-)*(\d.*)$} ) {    
 	    s{space-}{ }g;
 	    s{tab-}{\t}g;
@@ -412,13 +477,14 @@ sub make_response {
 	    $w =~s{cr}{\r}g;
 	    $w =~s{lf}{\n}g;
 	    $w =~s{\\([0-7]{3})}{ chr(oct($1)) }eg;
-	    push @transform, sub { $_[0] =~ s{(\r|\n|\r\n|\n\r)\1}{$w} or die }
+	    push @transform, sub { $_[0] =~ s{[\r\n]+\z}{$w} or die }
 	} elsif ( m{^proto:(.*)} ) {
 	    my $proto = $1;
 	    $proto =~s{cr|\\r}{\r}g;
 	    $proto =~s{tab|\\t}{\t}g;
 	    $proto =~s{lf|\\n}{\n}g;
 	    $proto =~s{space}{ }g;
+	    $proto =~s{\\([0-7]{3})}{ chr(oct($1)) }esg;
 	    $proto =~s{-}{}g;
 	    $statusline = "$proto $code ok\r\n";
 	} elsif ( m{^status:(.*)} ) {
@@ -471,15 +537,4 @@ sub make_response {
 }
 
 
-sub _compress {
-    my ($data,$w) = @_;
-    my $zlib = Compress::Raw::Zlib::Deflate->new(
-	-WindowBits => $w eq 'gzip' ? WANT_GZIP : -MAX_WBITS(),
-	-AppendOutput => 1,
-    );
-    my $newdata = '';
-    $zlib->deflate( $data, $newdata);
-    $zlib->flush($newdata,Z_FINISH);
-    return $newdata;
-}
 1;
